/* Calculate the altitude and azimuth of the Sun for a given time 
   and location

   Author: Dirk Terrell (terrell@falcon-net.net)
   Version of 11 September 1998

*/

/* Load math library and set up some constants */
call RxFuncAdd 'MathLoadFuncs', 'REXXMATH', 'MathLoadFuncs'
call MathLoadFuncs
Numeric Digits 16
Pi = atan(1.0)*4.0

/* Get input quantities */
Say "Enter the month (1=Jan, 2=Feb, etc.):"
Parse Pull Month

Say "Enter the day of the month:"
Parse Pull Day

Say "Enter the year:"
Parse Pull Year

Say "Enter the hour (UT):"
Parse Pull Hour

Say "Enter the minute:"
Parse Pull Minute

Say "Enter the second:"
Parse Pull Second

Time = (Hour + Minute/60 + Second/3600)/24

Say "Enter the longitude of the observer in degrees (west = positive):"
Parse Pull Longitude
LongitudeRadians = Deg2Rad(Longitude)

Say "Enter the latitude of the observer in degrees (north = positive):"
Parse Pull Latitude
LatitudeRadians = Deg2Rad(Latitude)

/* Calculate the Julian Date */
JD=JulianDate(Month, Day+Time, Year)
Say "Julian Date is" JD

T=(JD-2451545.0)/36525
T2=T*T
T3=T2*T

/* Compute the geometric mean longitude of the Sun */
GMLSun = Mod360(280.46645 + 36000.76983*T + 0.0003032*T2)

/* Calculate the mean anomaly of the Sun */
MASun = Mod360(357.5291 + 35999.05030*T - 0.0001559*T2 - 0.00000048*T3)
MASunRadians = Deg2Rad(MASun)
/* Calculate the eccentricity of the Earth's orbit */
Ecc = 0.016708617 - 0.000042037*T - 0.0000001236*T2

Say "Geometric Mean Longitude of Sun:" GMLSun
Say "Mean Anomaly of Sun:" MASun
Say "Eccentricity of Earth's Orbit:" Ecc

/* Get the equation of center of the Sun */
ECSun = (1.9146 - 0.004817*T - 0.000014*T2) * sin(MASunRadians),
      + (0.019993 - 0.000101*T) * sin(2*MASunRadians),
      + 0.00029 * sin(3*MASunRadians)
Say "Sun's Equation of Center:" ECSun

/* Sun's true geometric longitude ref. to the mean equinox of date */
TLSun = GMLSun + ECSun

/* Sun's apparent longitude i.e., include nutation and aberration */
Omega = Mod360(125.04 - 1934.136*T)
OmegaRadians = Deg2Rad(Omega)
Lambda = Mod360(TLSun - 0.00569 - 0.00478*Sin(OmegaRadians))
Say "Apparent longitude of Sun:" Lambda
LambdaRadians = Deg2Rad(Lambda)

/* Obliquity of the Ecliptic */
OblEcl = 23.43929111 - (46.815/3600)*T - (0.00059/3600)*T2 + (0.001813/3600)*T3
OblEcl = OblEcl + 0.00256*cos(OmegaRadians) /* Correction for apparent position */
OblEclRadians = Deg2Rad(OblEcl)

/* Get the RA and dec of the Sun */
RA = Rad2Deg(atan2(cos(OblEclRadians) * sin(LambdaRadians), cos(LambdaRadians)))
RA = Mod360(RA)
DecRadians = asin(sin(OblEclRadians) * sin(LambdaRadians))
Dec=Rad2Deg(DecRadians)

Say "Right ascension of Sun:" RA
Say "Declination of Sun:" Dec

/* Now rotate from the equatorial system to the local horizon system */

/* Get the Greenwich mean sidereal time 
   Technically, this should be the apparent sidereal time 
   (i.e., must include a correction for nutation
   but skipping it should be ok for the current purpose)
*/
Theta = Mod360(280.46061837 + 360.98564736629*(JD-2451545.0),
        + 0.000387933*T2 - T3/38710000)

Say "Greenwich mean siderial time:" Theta

/* Hour angle of the Sun */
HourAngle = Theta - RA - Longitude
HourAngleRadians = Deg2Rad(HourAngle)
Say "Sun's hour angle:" HourAngle

/* Azimuth reckoned eastward from north */
Azimuth = Rad2Deg(,
                  atan2(sin(HourAngleRadians), cos(HourAngleRadians)*sin(LatitudeRadians),
                  -tan(DecRadians)*cos(LatitudeRadians)),
                 ) + 180  
/* The 180 above makes azimuth start at north; use 0 if you want to start at south */
Say "Azimuth of Sun:" Azimuth

/* Altitude */
Altitude = Rad2Deg(,
                   asin(sin(LatitudeRadians)*sin(DecRadians),
                   + cos(LatitudeRadians)*cos(DecRadians)*cos(HourAngleRadians)))
Say "Altitude of Sun:" Altitude

Exit

Rad2Deg:
Procedure Expose Pi
/* Convert an input angle in radians to degrees */
Parse Arg Radians
Radians = Radians * (180/Pi)
Return Radians

Deg2Rad:
Procedure Expose Pi
/* Convert an input angle in degrees to radians */
Parse Arg Degrees
Radians = Degrees * (Pi/180)
Return Radians

Mod360:
Procedure
/* Return an input angle shifted to be between 0 and 360 degrees */
Parse Arg Angle
Remainder=Angle//360
If Remainder<0 then
   Remainder=Remainder+360
Return Remainder

JulianDate:
Procedure
Parse Arg Month , Day , Year
Say "Month:" Month
Say "Day:" Day
Say "Year:" Year

/* Calculate the Julian Date for date given in Day Month Year */
If Month<=2 then do
        Year=Year-1
        Month=Month+12
End
A=Floor(Year/100)
B=2-A+Floor(A/4)
JD=Floor(365.25*(Year+4716)) + Floor(30.6001*(Month+1)) + Day + B - 1524.5
Return JD

